/* 
 * 
 * This class is the thread that captures packets
 * 
 */
import net.sourceforge.jpcap.capture.PacketCapture;

public class CaptureThread extends Thread {

	/* references */
	private VPNmonitor vpn;
	private Traffic trafficPanel;
	private PlainText plaintextPanel;
	private Statistics statisticsPanel;

	private static final int INFINITY = -1;
	private static final int PACKET_COUNT = INFINITY;

	private PacketCapture pcap;

	public CaptureThread(
										Traffic traffic,
										PlainText plaintext,
										Statistics statistics,
										VPNmonitor vpn) {
		super();
		trafficPanel = traffic;
		plaintextPanel = plaintext;
		statisticsPanel = statistics;
		this.vpn = vpn;
	}

	public void run() {

		try {
			trafficPanel.startButton.setEnabled(false);
			trafficPanel.stopButton.setEnabled(true);
			plaintextPanel.getNewModel(); /* make sure that the plaintext panel has a new table */
			
			pcap = new PacketCapture();

			String usingDevice =
				(String) trafficPanel.deviceCombo.getSelectedItem();

			System.out.println("Using device: " + usingDevice);
			pcap.open(usingDevice, 1500, true, -1); // 1500 is the MTU
			//pcap.open(usingDevice, true); // promiscuous mode
			final String FILTER =
				// "src 142.103.242.131";
				""; // no filter
			pcap.setFilter(FILTER, true);
			pcap.addPacketListener(
													new PacketHandler(
														trafficPanel,
														plaintextPanel,
														statisticsPanel));

			while (vpn.captureThreadisStopped == false)
			{
				pcap.capture(1);
			}
			
			trafficPanel.startButton.setEnabled(true);
			trafficPanel.stopButton.setEnabled(false);
		} catch (Exception e) {
			System.out.println(e.getStackTrace());
		}
	} /* end of run */
} /* end of class */
