/* 
 * This class carries out operations when a packet is received
 */
import net.sourceforge.jpcap.net.*;
import net.sourceforge.jpcap.capture.*;

class PacketHandler implements PacketListener {

	/* references for convinence */
	private static Traffic trafficPanel;
	private static PlainText plaintextPanel;
	private static Statistics statisticsPanel;

	public PacketHandler(
										Traffic traffic,
										PlainText plaintext,
										Statistics statistics) {

		/* get the references for easy access */
		trafficPanel = traffic;
		plaintextPanel = plaintext;
		statisticsPanel = statistics;
	}

	/* when a packet arrives... */
	public void packetArrived(Packet packet) {

		/* we only care about IP packets */
		if (packet instanceof IPPacket) {

			IPPacket ipPacket = (IPPacket) packet;
			int type = Traffic.PLAINTEXT;
			if (ipPacket.getProtocol() == 0x06) {
				/* a TCP packet has arrived */
				TCPPacket tcpPacket = (TCPPacket) packet;
				
				if (tcpPacket.getDestinationPort() == 443) {
					/* a TCP-HTTPS packet has arrived */
					type = Traffic.HTTPS;
					statisticsPanel.numHTTPS++;
				} else {
					/* it is a TCP packet, but not a HTTPS packet */
					statisticsPanel.numOthers++;
				}
				
			} else if (ipPacket.getProtocol() == 0x2f) {
				/* a GRE packet has arrived 
				 * a GRE packet is used by a PPTP connection */
				type = Traffic.PPTP;
				statisticsPanel.numPPTP++;
				
			} else if (ipPacket.getProtocol() == 0x32) {
                /* an ESP packet has arrived
                 * an ESP packet is used by an IPSec connection */
				type = Traffic.IPSEC;
				statisticsPanel.numIPSEC++;

			} else {
                /* It is not PPTP nor IPSec packet */
				statisticsPanel.numOthers++;
			}

			/* add the pair of the connection */
			trafficPanel.AddVPNPair(
				ipPacket.getSourceAddress(),
				ipPacket.getDestinationAddress(),
				type);

			/* dump the text if it is not encrypted */
			if (type == Traffic.PLAINTEXT) {
				plaintextPanel.DumpAddresses(packet);
			}

		} else {

			/* It is not an IP packet */
			statisticsPanel.numOthers++;
		}
	}
}
