/*
 * This class is responsible for the panel of Plane Text
 */
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import net.sourceforge.jpcap.net.Packet;
import net.sourceforge.jpcap.net.IPPacket;
import net.sourceforge.jpcap.net.*;
import net.sourceforge.jpcap.util.*;

public class PlainText extends JPanel implements ActionListener {

	private static JTable table;
	private static JScrollPane scrollPane;
	private DefaultTableModel model;
	public DefaultTableModel savedModel = null;
	private static JButton filterButton;
	private static JTextField filterText;
	private static JButton saveButton;
	private static JButton loadButton;

	public PlainText() {
		model = new DefaultTableModel();
		table = new JTable(model);

		// Create a couple of columns
		model.addColumn("ID");
		model.addColumn("Source");
		model.addColumn("Destination");
		model.addColumn("Text");

		/* allow a text to span multiple lines */
		table.getColumnModel().getColumn(3).setCellRenderer(
			new MultiLineCellRenderer());

		//Create the scroll pane and add the table to it. 
		scrollPane = new JScrollPane(table);

		setColumnWidth();

		filterButton = new JButton("Filter");
		filterButton.setActionCommand("filter");
		filterButton.addActionListener(this);

		filterText = new JTextField();

		// add button to a panel, make it align to the left
		JPanel buttonPane = new JPanel();
		buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.X_AXIS));
		buttonPane.add(filterButton);
		buttonPane.add(filterText);

		// add save/load buttons
		JPanel bottomPane = new JPanel();
		saveButton = new JButton("Save");
		saveButton.setActionCommand("save");
		saveButton.addActionListener(this);
		loadButton = new JButton("Load");
		loadButton.setActionCommand("load");
		loadButton.addActionListener(this);
		bottomPane.setLayout(new BoxLayout(bottomPane, BoxLayout.X_AXIS));
		bottomPane.add(saveButton);
		bottomPane.add(loadButton);

		// add everything to this panel
		setLayout(new BorderLayout());
		add(buttonPane, BorderLayout.NORTH);
		add(scrollPane, BorderLayout.CENTER);
		add(bottomPane, BorderLayout.SOUTH);
	}

	/* main function that receives a packet and dump the text to the screen */
	public void DumpAddresses(Packet packet) {
		IPPacket ipPacket = (IPPacket) packet;
		//String aString = Integer.toString(ipPacket.getLength()) + ", " 
		//                         + Integer.toString(ipPacket.getHeaderLength());
		
		String data = AsciiHelper.toString(packet.getData());
		model.insertRow(
			0,
			new Object[] {
				Integer.toString(model.getRowCount() + 1),
				ipPacket.getSourceAddress(),
				ipPacket.getDestinationAddress(),
				data});
	}
	
	public void DumpData(RawPacket rawPacket)
	{
		//String data = AsciiHelper.toString(rawPacket.getData());
		//model.setValueAt(new Object[]{data}, 0,3);
		//System.out.println("----- ");
		//System.out.println(data);	
	}

	public void actionPerformed(ActionEvent e) {
		if ("filter".equals(e.getActionCommand())) {
			runFilter();
		} else {

			if ("save".equals(e.getActionCommand())) {
				runSave();
			} else {

				if ("load".equals(e.getActionCommand())) {
					runLoad();
				}
			}
		}
	}

	public void getNewModel() {
		// set up the newModel
		model = new DefaultTableModel();
		model.addColumn("ID");
		model.addColumn("Source");
		model.addColumn("Destination");
		model.addColumn("Text");
		table.setModel(model);

		table.getColumnModel().getColumn(3).setCellRenderer(
			new MultiLineCellRenderer());

		setColumnWidth();
	}

	private void runFilter() {
		// set up the newModel
		DefaultTableModel newModel = new DefaultTableModel();
		newModel.addColumn("ID");
		newModel.addColumn("Source");
		newModel.addColumn("Destination");
		newModel.addColumn("Text");

		String words[] = new String[100];
		int wordCount = 0;

		// remove trailing space and ,,,,,
		String filterWords = filterText.getText().trim();
		while (filterWords
			.substring(filterWords.length() - 1, filterWords.length())
			.compareTo(",")
			== 0) {
			filterWords =
				filterWords.substring(0, filterWords.length() - 1).trim();
		}

		int oldIndex = -1;
		int newIndex = -1;

		boolean repeat = true;
		while (repeat) {
			newIndex = filterWords.indexOf(",", oldIndex + 1);
			if (newIndex != -1) {
				String newWord =
					((filterWords.substring((oldIndex + 1), newIndex))).trim();
				if (newWord.length() != 0) {
					words[wordCount] = newWord;
					wordCount++;
				}
				oldIndex = newIndex;
			} else {
				String newWord =
					(filterWords
						.substring((oldIndex + 1), filterWords.length()))
						.trim();
				if (newWord.length() != 0) {
					words[wordCount] = newWord;
					wordCount++;
					repeat = false;
				}
			}
		}

		/* now all words are in the "words" array */
		/* for debug:
		System.out.println("Words are:");
		for (int i = 0; i < wordCount; i++) {
			System.out.println(words[i]);
		}
		*/

		for (int i = 0; i < model.getRowCount(); i++) {
			String text = (String) model.getValueAt(i, 3);

			boolean hasAword = false;
			for (int wordIndex = 0;(wordIndex < wordCount); wordIndex++) {
				if (text.indexOf(words[wordIndex]) != -1) {
					hasAword = true;
				}
			}

			if (hasAword) {
				// contains the substring
				newModel.addRow(
					new Object[] {
						model.getValueAt(i, 0),
						model.getValueAt(i, 1),
						model.getValueAt(i, 2),
						model.getValueAt(i, 3)});
			}
		}
		table.setModel(newModel);
		setColumnWidth();

		table.getColumnModel().getColumn(3).setCellRenderer(
			new MultiLineCellRenderer());

	}

	private void runSave() {
		System.out.println("runSave");
		savedModel = (DefaultTableModel) table.getModel();
	}

	public void runLoad() {
		System.out.println("runLoad");
		if (savedModel != null) {
			table.setModel(savedModel);
			setColumnWidth();
		} else {
			JOptionPane.showMessageDialog(
				null,
				"Nothing was saved",
				"Error",
				JOptionPane.OK_OPTION);
		}
	}

	private void setColumnWidth() {
		//			set column width
		TableColumn column = null;
		for (int i = 0; i < 4; i++) {
			column = table.getColumnModel().getColumn(i);
			if (i == 0) {
				column.setPreferredWidth(10);
			} else if (i == 3) {
				column.setPreferredWidth(500);
			} else {
				column.setPreferredWidth(100);
			}
		}
	}
}
