/*
 * This class keeps all the statistics of connections and packets
 */
import java.awt.BorderLayout;
import java.awt.Graphics;
import java.util.Iterator;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;


public class Statistics extends JPanel {

	public int numIPSEC;
	public int numPPTP;
	public int numHTTPS;
	public int numOthers;
	private int numTotal;

	private JPanel packetStat;
	private JLabel numIPSECLabel;
	private JLabel numPPTPLabel;
	private JLabel numHTTPSLabel;
	private JLabel numOthersLabel;
	private JLabel numTotalLabel;

	private JPanel connectionStat;
	private JLabel IPSECconnectionLabel;
	private JLabel PPTPconnectionLabel;
	private JLabel HTTPSconnectionLabel;
	private JLabel OthersconnectionLabel;
	private JLabel TotalconnectionLabel;
	private int IPSECconnections;
	private int PPTPconnections;
	private int HTTPSconnections;
	private int Othersconnections;
	private int Totalconnections;
	
	private JPanel packetConnectionPanel;
	
	private JPanel connections;
	private JTable table;
	public DefaultTableModel model;
	private JScrollPane scrollPane;

	private Traffic trafficPanel;

	public Statistics(Traffic aTrafficPanel) {
		trafficPanel = aTrafficPanel;
		trafficPanel.statisticsPanel = this;

		setLayout(new BorderLayout());

		/*
		 * Packet Statistic Panel
		 */
		packetStat = new JPanel();
		packetStat.setLayout(new BoxLayout(packetStat, BoxLayout.Y_AXIS));
		packetStat.setBorder(
			BorderFactory.createTitledBorder("Packet Statistics"));

		numIPSECLabel = new JLabel();
		numPPTPLabel = new JLabel();
		numHTTPSLabel = new JLabel();
		numOthersLabel = new JLabel();
		numTotalLabel = new JLabel();
		refreshPacketStatistics();
		packetStat.add(numIPSECLabel);
		packetStat.add(numPPTPLabel);
		packetStat.add(numHTTPSLabel);
		packetStat.add(numOthersLabel);
		packetStat.add(numTotalLabel);

		/*
		 * Connection Statistic Panel
		 */
		connectionStat = new JPanel();
		connectionStat.setLayout(
			new BoxLayout(connectionStat, BoxLayout.Y_AXIS));
		connectionStat.setBorder(
			BorderFactory.createTitledBorder("Connection Statistics"));

		IPSECconnectionLabel = new JLabel();
		PPTPconnectionLabel = new JLabel();
		HTTPSconnectionLabel = new JLabel();
		OthersconnectionLabel = new JLabel();
		TotalconnectionLabel = new JLabel();
		refreshConnectionStatistics();
		
		connectionStat.add(IPSECconnectionLabel);
		connectionStat.add(PPTPconnectionLabel);
		connectionStat.add(HTTPSconnectionLabel);
		connectionStat.add(OthersconnectionLabel);
		connectionStat.add(TotalconnectionLabel);
		
		packetConnectionPanel = new JPanel();
		packetConnectionPanel.setLayout(new BorderLayout());
		packetConnectionPanel.add(packetStat, BorderLayout.NORTH);
		packetConnectionPanel.add(connectionStat, BorderLayout.CENTER);
		
		/*
		 * connections
		 */
		connections = new JPanel();
		model = new DefaultTableModel();
		table = new JTable(model);
    
		model.addColumn("Host 1");
		model.addColumn("Host 2");
		model.addColumn("Connection Type");

		scrollPane = new JScrollPane(table);
		add(scrollPane, BorderLayout.CENTER);
		add(packetConnectionPanel, BorderLayout.WEST);
	}

	public void paint(Graphics g) {
		refreshPacketStatistics();
		refreshConnectionStatistics();

		packetConnectionPanel.repaint();
		scrollPane.repaint();
	}

	/* recalcualte the packet statistics */
	public void refreshPacketStatistics() {

		int total = numIPSEC + numPPTP + numHTTPS + numOthers;
		numTotal = total;
		if (total == 0)
			total = 1;
		numIPSECLabel.setText(
			"IPSec Packets: "
				+ numIPSEC
				+ " ( "
				+ (numIPSEC * 100 / total)
				+ " % )");
		numPPTPLabel.setText(
			"PPTP Packets: "
				+ numPPTP
				+ " ( "
				+ (numPPTP * 100 / total)
				+ " % )");
		numHTTPSLabel.setText(
			"HTTPS Packets: "
				+ numHTTPS
				+ " ( "
				+ (numHTTPS * 100 / total)
				+ " % )");
		numOthersLabel.setText(
			"Other Packets: "
				+ numOthers
				+ " ( "
				+ (numOthers * 100 / total)
				+ " % )");
		numTotalLabel.setText(
			"Total Packets: "
				+ numTotal
				+ " ( "
				+ (numTotal * 100 / total)
				+ " % )");
	}

	/* recalculate connection statistics */
	public void refreshConnectionStatistics() {
		IPSECconnections = 0;
		PPTPconnections = 0;
		HTTPSconnections = 0;
		Othersconnections = 0;
		Totalconnections = 0;

		Iterator listIterator = trafficPanel.VPNPairList.listIterator();
		while (listIterator.hasNext()) {
			Pair aPair = (Pair) listIterator.next();
			if (aPair.type == Traffic.IPSEC) {
				IPSECconnections++;
			} else if (aPair.type == Traffic.PPTP) {
				PPTPconnections++;
			} else if (aPair.type == Traffic.HTTPS) {
				HTTPSconnections++;
			} else {
				Othersconnections++;
			}
			Totalconnections++;
		}

		int ctotal =
			IPSECconnections
				+ PPTPconnections
				+ HTTPSconnections
				+ Othersconnections;
		if (ctotal == 0)
			ctotal = 1;
		IPSECconnectionLabel.setText(
			"IPSEC Connections: "
				+ IPSECconnections
				+ " ( "
				+ (IPSECconnections * 100 / ctotal)
				+ " % )");
		PPTPconnectionLabel.setText(
			"PPTP Connections: "
				+ PPTPconnections
				+ " ( "
				+ (PPTPconnections * 100 / ctotal)
				+ " % )");
		HTTPSconnectionLabel.setText(
			"HTTPS Connections: "
				+ HTTPSconnections
				+ " ( "
				+ (HTTPSconnections * 100 / ctotal)
				+ " % )");
		OthersconnectionLabel.setText(
			"Other Connections: "
				+ Othersconnections
				+ " ( "
				+ (Othersconnections * 100 / ctotal)
				+ " % )");
		TotalconnectionLabel.setText(
			"Total Connections: "
				+ Totalconnections
				+ " ( "
				+ (Totalconnections * 100 / ctotal)
				+ " % )");
	}
}
