/*
 * This class is to display the traffic and connection graphically
 */
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Random;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.sourceforge.jpcap.capture.PacketCapture;

public class Traffic
	extends JPanel
	implements ActionListener, MouseMotionListener {

	public Statistics statisticsPanel;
	// initialize when statisticsPanel is initialized
	public LinkedList VPNPairList;
	public LinkedList NodeList;
	private Hashtable NodeRealLocation;

	Random random;
	private static int radius = 180;

	private JLabel deviceLabel;

	public JComboBox deviceCombo;
	private static JButton rearrangeButton;
	public JButton startButton;
	public JButton stopButton;

	private static JPanel leftPanel;
	private static JPanel controls;
	private static JPanel legend;

	public static JPanel computerPanel;

	public static final int PLAINTEXT = 0;
	public static final int IPSEC = 1;
	public static final int PPTP = 2;
	public static final int HTTPS = 3;

	public Traffic(int Width, int Height) {
		super(false);
		VPNPairList = new LinkedList();
		NodeList = new LinkedList();
		NodeRealLocation = new Hashtable();

		random = new Random();

		setLayout(new BorderLayout());

		leftPanel = new JPanel();
		leftPanel.setLayout(new GridLayout(0, 1));

		deviceLabel = new JLabel("Device :");
		String[] devices = null;
		try {
			devices = PacketCapture.lookupDevices();
		} catch (Exception e) {
			e.printStackTrace();
		}
		deviceCombo = new JComboBox(devices);
		//	deviceCombo.setSelectedIndex(0);

		rearrangeButton = new JButton("Re-arrange Computers");
		rearrangeButton.setActionCommand("rearrange");
		rearrangeButton.addActionListener(this);
		startButton = new JButton("Start Monitoring");
		stopButton = new JButton("Stop Monitoring");
		stopButton.setEnabled(false);

		controls = new JPanel();
		//		controls.setLayout(new BoxLayout(controls, BoxLayout.Y_AXIS));
		controls.setLayout(new GridLayout(0, 1));
		controls.add(deviceLabel);
		controls.add(deviceCombo);
		controls.add(startButton);
		controls.add(stopButton);
		controls.add(rearrangeButton);
		controls.add(Box.createRigidArea(new Dimension(0, 80)));
		controls.setBorder(BorderFactory.createTitledBorder("Operations"));

		leftPanel.add(controls);

		JLabel ipSecLabel = new JLabel("  IPSec ----");
		ipSecLabel.setForeground(Color.BLUE);
		JLabel pptpLabel = new JLabel("  PPTP ----");
		pptpLabel.setForeground(Color.GREEN);
		JLabel httpsLabel = new JLabel("  HTTPS ----");
		httpsLabel.setForeground(Color.ORANGE);
		legend = new JPanel();
		legend.setLayout(new BoxLayout(legend, BoxLayout.Y_AXIS));
		legend.add(ipSecLabel);
		legend.add(pptpLabel);
		legend.add(httpsLabel);
		legend.setBorder(BorderFactory.createTitledBorder("Legend"));
		leftPanel.add(legend);

		add(leftPanel, BorderLayout.WEST);

		computerPanel = new JPanel();
		//computerPanel.setLayout(new GridLayout(8,8));
		computerPanel.setLayout(null);
		computerPanel.setBorder(BorderFactory.createTitledBorder("Network"));

		add(computerPanel, BorderLayout.CENTER);

	}

	public void paint(Graphics g) {
		super.paint(g);
		drawLines(g);
	}

	private void drawLines(Graphics g) {

		Iterator listIterator = VPNPairList.listIterator();
		while (listIterator.hasNext()) {
			Pair aPair = (Pair) listIterator.next();
			String from = aPair.from;
			String to = aPair.to;

			if (aPair.type == IPSEC) {
				g.setColor(Color.BLUE);
			} else if (aPair.type == PPTP) {
				g.setColor(Color.GREEN);
			} else if (aPair.type == HTTPS) {
				g.setColor(Color.ORANGE);
			} else {
				g.setColor(Color.BLACK);
			}

			Point fromPoint = (Point) NodeRealLocation.get(from.toString());
			Point toPoint = (Point) NodeRealLocation.get(to.toString());

			int indent = computerPanel.getLocation().x;
			g.drawLine(
				fromPoint.x + indent + 100,
				fromPoint.y + 30,
				toPoint.x + indent + 100,
				toPoint.y + 30);
		}
	}

	public boolean AddVPNPair(String from, String to, int type) {

		boolean gotNewNode = false;

		Node fromNode = new Node(from);
		Node toNode = new Node(to);
		Pair newPair = new Pair(from, to, type);

		int x, y;
		// check if the fromNode is already in NodeList
		if (NodeListContainsNode(NodeList, fromNode)) {
		} else {

			x = getx();
			y = gety(x);
			fromNode.x = x;
			fromNode.y = y;

			NodeList.add(fromNode);
			addNodeToPanel(fromNode);
			gotNewNode = true;
		}

		if (NodeListContainsNode(NodeList, toNode)) {
		} else {
			x = getx();
			y = gety(x);
			toNode.x = x;
			toNode.y = y;
			NodeList.add(toNode);
			addNodeToPanel(toNode);
			gotNewNode = true;
		}

		Pair aPair;

		// if the pair contains at least one new node
		// then it must be a new pair
		if (gotNewNode) {
			VPNPairList.add(newPair);

			String connectionType;
			switch (newPair.type) {
				case (Traffic.IPSEC) :
					connectionType = "IPsec";
					break;
				case (Traffic.HTTPS) :
					connectionType = "HTTPS";
					break;
				case (Traffic.PPTP) :
					connectionType = "PPTP";
					break;
				case (Traffic.PLAINTEXT) :
					connectionType = "unencrypted";
					break;
				default :
					connectionType = "undefined";
					break;
			}

			statisticsPanel.model.addRow(
				new Object[] { newPair.from, newPair.to, connectionType });
			return true;
		}

		// if both node already exists, check if the pair
		// already exists, if not, then add it
		aPair = PairListContainsPair(VPNPairList, newPair);
		if (aPair == null) {
			VPNPairList.add(newPair);

			String connectionType;
			switch (newPair.type) {
				case (Traffic.IPSEC) :
					connectionType = "IPsec";
					break;
				case (Traffic.HTTPS) :
					connectionType = "HTTPS";
					break;
				case (Traffic.PPTP) :
					connectionType = "PPTP";
					break;
				case (Traffic.PLAINTEXT) :
					connectionType = "unencrypted";
					break;
				default :
					connectionType = "undefined";
					break;
			}
			statisticsPanel.model.addRow(
				new Object[] { newPair.from, newPair.to, connectionType });
			return true;
		} else
			// if both node already exists, and the pair also exists
			// then update its connection type
			{
			int numRows = statisticsPanel.model.getRowCount();
			for (int i = 0; i < numRows; i++) {
				if (((String) statisticsPanel.model.getValueAt(i, 0))
					.compareTo(aPair.from)
					== 0) {
					if (((String) statisticsPanel.model.getValueAt(i, 1))
						.compareTo(aPair.to)
						== 0) {
						// connections only upgrade, won't downgrade
						if (((String) statisticsPanel.model.getValueAt(i, 2))
							.compareTo(Integer.toString(0))
							== 0) {
							// old entry found, update!
							String connectionType;
							switch (newPair.type) {
								case (Traffic.IPSEC) :
									connectionType = "IPsec";
									break;
								case (Traffic.HTTPS) :
									connectionType = "HTTPS";
									break;
								case (Traffic.PPTP) :
									connectionType = "PPTP";
									break;
								case (Traffic.PLAINTEXT) :
									connectionType = "unencrypted";
									break;
								default :
									connectionType = "undefined";
									break;
							}
							statisticsPanel.model.setValueAt(
								connectionType,
								i,
								2);
						}
					}
				}
			}
			aPair.updateType(newPair);
			return true;
		}
	}

	private void addNodeToPanel(Node aNode) {
		ImageIcon icon = createImageIcon("images/Server.gif", "");

		JLabel aComputer = new JLabel(aNode.ip.toString(), icon, JLabel.CENTER);
		//  Set the position of the text, relative to the icon:
		aComputer.setVerticalTextPosition(JLabel.TOP);
		aComputer.setHorizontalTextPosition(JLabel.CENTER);
		//aComputer.setBounds(aNode.x - 100, aNode.y - 35, 200, 50);
		//aComputer.setBounds(0, 0, 200, 50);
		aComputer.setBounds(
			aNode.x + computerPanel.getWidth() / 2 - 100,
			aNode.y + computerPanel.getHeight() / 2 - 30,
			200,
			50);

		aComputer.setForeground(Color.MAGENTA);
		computerPanel.add(aComputer);
		computerPanel.validate();

		NodeRealLocation.put(aNode.ip.toString(), aComputer.getLocation());

		aComputer.addMouseMotionListener(this);

	}

	private int getx() {

		int x;
		x = random.nextInt(radius);

		if (random.nextDouble() < 0.5)
			x = -1 * x;
		//x = x + centerX;
		return x;
	}

	private int gety(int x) {

		//x = x - centerX;

		int y;
		y = (int) java.lang.Math.sqrt((double) (radius * radius - x * x));

		if (random.nextDouble() < 0.5)
			y = -1 * y;
		//y = y + centerY;

		return y;
	}

	private boolean NodeListContainsNode(LinkedList aNodeList, Node aNode) {
		Iterator listIterator = aNodeList.listIterator();

		while (listIterator.hasNext()) {
			Node currentNode = (Node) listIterator.next();
			if (currentNode.equals(aNode))
				return true;
		}
		return false;

	}

	private int NodeListx(LinkedList aNodeList, Node aNode) {
		Iterator listIterator = aNodeList.listIterator();
		while (listIterator.hasNext()) {
			Node currentNode = (Node) listIterator.next();
			if (currentNode.equals(aNode))
				return currentNode.x;
		}
		System.err.println("Error on X!");
		return -1;
	}

	private int NodeListy(LinkedList aNodeList, Node aNode) {
		Iterator listIterator = aNodeList.listIterator();
		while (listIterator.hasNext()) {
			Node currentNode = (Node) listIterator.next();
			if (currentNode.equals(aNode))
				return currentNode.y;
		}
		System.err.println("Error on Y!");
		return -1;
	}

	private Pair PairListContainsPair(LinkedList PairList, Pair aPair) {
		Iterator listIterator = PairList.listIterator();

		Pair foundPair;
		while (listIterator.hasNext()) {
			Pair currentPair = (Pair) listIterator.next();
			if (currentPair.equals(aPair)) {
				foundPair = currentPair;
				return foundPair;
			}
		}
		return null;
	}

	/** Returns an ImageIcon, or null if the path was invalid. */
	/* Copied from Java homepage */
	protected static ImageIcon createImageIcon(
		String path,
		String description) {
		java.net.URL imgURL = Traffic.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL, description);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}

	public void actionPerformed(ActionEvent e) {
		if ("rearrange".equals(e.getActionCommand())) {
			RearrangeComputers();
		} else {
		}
	}

	public void RearrangeComputers() {
		computerPanel.removeAll();

		Iterator listIterator = NodeList.listIterator();
		while (listIterator.hasNext()) {
			Node currentNode = (Node) listIterator.next();
			currentNode.x = getx();
			currentNode.y = gety(currentNode.x);

			addNodeToPanel(currentNode);
		}
		repaint();
	}

	public void mouseDragged(MouseEvent arg0) {

		if (arg0.getSource() instanceof JLabel) {
			JLabel theLabel = (JLabel) arg0.getSource();

			Point location = theLabel.getLocation();
			int gCenterX = location.x + arg0.getX();
			int gCenterY = location.y + arg0.getY();

			location.x = gCenterX - theLabel.getWidth() / 2;
			location.y = gCenterY - theLabel.getHeight() / 2;

			theLabel.setLocation(location);

			NodeRealLocation.put(theLabel.getText(), location);

			repaint();
		}

	}

	public void mouseMoved(MouseEvent arg0) {
	}

}
