/*
 * main class
 */
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.Timer;

public class VPNmonitor extends JPanel implements ActionListener {

	// GUI components
	private JTabbedPane tabbedPane;
	private Traffic trafficPanel;
	private PlainText plaintextPanel;
	private Statistics statisticsPanel;

	// GUI parameters
	final private static int WIDTH = 800;
	final private static int HEIGHT = 500;
	
	public boolean captureThreadisStopped;
	
	public VPNmonitor() {
		
		tabbedPane = new JTabbedPane();

		plaintextPanel = new PlainText();
		trafficPanel = new Traffic(WIDTH, HEIGHT);
		statisticsPanel = new Statistics(trafficPanel);

		tabbedPane.addTab("Traffic", trafficPanel);
		tabbedPane.addTab("Plain Text", plaintextPanel);
		tabbedPane.addTab("Statistics", statisticsPanel);
		tabbedPane.setSelectedIndex(0);

		//Add the tabbed pane to this panel.
		setLayout(new GridLayout(1, 1));
		add(tabbedPane);
		
		trafficPanel.startButton.setActionCommand("start");
		trafficPanel.stopButton.setActionCommand("stop");
		trafficPanel.startButton.addActionListener(this);
		trafficPanel.stopButton.addActionListener(this);
	}
	
	public static void main(String[] args) throws java.io.IOException {

		JFrame VPNmonitor = new JFrame("VPNmonitor");
		VPNmonitor.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		
		VPNmonitor.getContentPane().add(new VPNmonitor(), BorderLayout.CENTER);
		VPNmonitor.setSize(WIDTH, HEIGHT);
		VPNmonitor.setVisible(true);
	}

	public static int unsignedByteToInt(byte b) {
		return (int) b & 0xFF;
	}
	
	public void actionPerformed(ActionEvent e) {
		if ("start".equals(e.getActionCommand())) {
			captureThreadisStopped = false;
			run();
			trafficPanel.repaint();
		} else if ("stop".equals(e.getActionCommand())) {
			captureThreadisStopped = true;
		}
	}
	
	private void run()
	{
		new CaptureThread(trafficPanel, plaintextPanel, statisticsPanel, this).start();
		
		Timer timer = new Timer(500, new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				trafficPanel.repaint();
				//plaintextPanel.repaint();
				statisticsPanel.refreshPacketStatistics();
				statisticsPanel.refreshConnectionStatistics();
			}
		});
		timer.start();
		
	}

}
